@@ -21,8 +21,9 @@
 
 package Lintian::binaries;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Spec;
@@ -31,7 +32,7 @@ use List::MoreUtils qw(any);
 use Lintian::Data;
 use Lintian::Relation qw(:constants);
 use Lintian::Spelling qw(check_spelling);
-use Lintian::Util qw(internal_error strip);
+use Lintian::Util qw(strip);
 
 use constant NUMPY_REGEX => qr/
     \Qmodule compiled against ABI version \E (?:0x)?%x
@@ -88,15 +89,15 @@ sub _embedded_libs {
             } elsif ($opt eq 'source-regex') {
                 $result->{$opt} = qr/$val/;
             } else {
-                internal_error("Unknown option $opt used for $key"
-                      . ' (in binaries/embedded-libs)');
+                die
+"Unknown option $opt used for $key (in binaries/embedded-libs)";
             }
         }
     }
 
     if (defined $result->{'source'} and $result->{'source-regex'}) {
-        internal_error("Both source and source-regex used for $key"
-              . ' (in binaries/embedded-libs)');
+        die
+"Both source and source-regex used for $key (in binaries/embedded-libs)";
     } else {
         $result->{'source'} = $key unless defined $result->{'source'};
     }
@@ -128,7 +129,7 @@ our $OBSOLETE_CRYPT_FUNCTIONS
 
 our $ARCH_32_REGEX;
 
-sub always {
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
@@ -375,8 +376,7 @@ sub always {
             foreach my $obj (@{ $objdump->{'objects'} }) {
                 my $libobj = $processable->objdump_info->{"${file}(${obj})"};
                 # Shouldn't happen, but...
-                internal_error(
-                    "object ($file $obj) in static lib is missing!?")
+                die "object ($file $obj) in static lib is missing!?"
                   unless defined $libobj;
 
                 if (any { exists($libobj->{'SH'}{$_}) } DEBUG_SECTIONS) {
@@ -464,8 +464,8 @@ sub always {
             # stripped but a debug or profiling library?
             if (($fname =~ m,/lib/debug/,o) or ($fname =~ m,/lib/profile/,o)){
                 $self->tag(
-                    'library-in-debug-or-profile-should-not-be-stripped',
-                    $file);
+                    'library-in-debug-or-profile-should-not-be-stripped',$file)
+                  unless $file->size == 0;
             } else {
                 # appropriately stripped, but is it stripped enough?
                 $self->tag_unneeded_sections('binary-has-unneeded-section',
