@@ -24,8 +24,9 @@
 
 package Lintian::fields::package_relations;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Dpkg::Version qw(version_check);
@@ -99,15 +100,7 @@ our $OBSOLETE_PACKAGES
   = Lintian::Data->new('fields/obsolete-packages',qr/\s*=>\s*/);
 our $VIRTUAL_PACKAGES   = Lintian::Data->new('fields/virtual-packages');
 
-sub udeb {
-    my ($self) = @_;
-
-    $self->binary;
-
-    return;
-}
-
-sub binary {
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
@@ -174,8 +167,9 @@ sub binary {
                 && $known_obsolete_emacs{$alternatives[0][0]});
 
             for my $part_d (@alternatives) {
-                my ($d_pkg, undef, $d_version, undef, undef, $rest,
-                    $part_d_orig)= @$part_d;
+                my ($d_pkg, $d_march, $d_version, undef, undef, $rest,
+                    $part_d_orig)
+                  = @$part_d;
 
                 $self->tag('invalid-versioned-provides', $part_d_orig)
                   if ( $field eq 'provides'
@@ -204,6 +198,7 @@ sub binary {
 
                 $self->tag('package-relation-with-self',"$field: $part_d_orig")
                   if ($pkg eq $d_pkg)
+                  && (!$d_march)
                   && ( $field ne 'conflicts'
                     && $field ne 'replaces'
                     && $field ne 'provides');
@@ -232,6 +227,13 @@ sub binary {
                     && $d_pkg ne 'diffutils'
                     && $d_pkg ne 'dash');
 
+                $self->tag('package-depends-on-an-x-font-package',
+                    "$field: $part_d_orig")
+                  if ( $field =~ /^(?:pre-)?depends$/
+                    && $d_pkg =~ /^xfont.*/
+                    && $d_pkg ne 'xfonts-utils'
+                    && $d_pkg ne 'xfonts-encodings');
+
                 $self->tag('depends-on-packaging-dev',$field)
                   if (($field =~ /^(?:pre-)?depends$/|| $field eq 'recommends')
                     && $d_pkg eq 'packaging-dev'
