@@ -20,8 +20,9 @@
 
 package Lintian::files::privacy_breach;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Lintian::SlidingWindow;
@@ -90,7 +91,7 @@ sub detect_privacy_breach {
     return
       unless $file->is_regular_file;
 
-    my $fd = $file->open(':raw');
+    open(my $fd, '<:raw', $file->unpacked_path);
 
     my $sfd = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); },BLOCKSIZE);
 
@@ -271,8 +272,7 @@ sub check_tag_url_privacy_breach {
     # followed automatically.
     if(    $file->basename =~ '.xml$'
         && $tagattr eq 'link'
-        && $file->file_contents
-        =~ qr{ xmlns="http://projectmallard\.org/1\.0/"}) {
+        && $file->bytes=~ qr{ xmlns="http://projectmallard\.org/1\.0/"}) {
         return;
     }
 
@@ -307,7 +307,7 @@ sub files {
     if (   $file->is_file
         && $file->name =~ m,\.(?:x?html?\d?|js|xht|xml|css)$,i) {
 
-        if(     $self->processable->pkg_src eq 'josm'
+        if(     $self->processable->source eq 'josm'
             and $file->basename eq 'defaultpresets.xml') {
             # false positive
 
