@@ -20,8 +20,9 @@
 
 package Lintian::shared_libs;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Basename;
@@ -29,7 +30,7 @@ use List::MoreUtils qw(any none uniq);
 
 use Lintian::Data;
 use Lintian::Relation;
-use Lintian::Util qw(internal_error strip);
+use Lintian::Util qw(strip);
 
 use Moo;
 use namespace::clean;
@@ -53,7 +54,7 @@ my %symbols_meta_fields = map { $_ => 1 }qw(
 my $ldconfig_dirs = Lintian::Data->new('shared-libs/ldconfig-dirs');
 my $MA_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/);
 
-sub always {
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
@@ -215,8 +216,7 @@ sub always {
     for my $shlib_file (keys %SONAME) {
         # file found?
         if (not $processable->installed->lookup($shlib_file)) {
-            internal_error(
-                "shlib $shlib_file not found in package (should not happen!)");
+            die "shlib $shlib_file not found in package (should not happen!)";
         }
 
         my ($dir, $shlib_name) = $shlib_file =~ m,(.*)/([^/]+)$,;
@@ -672,7 +672,7 @@ sub always {
     # 6th step: check pre- and post- control files
     if (my $preinst = $processable->control->resolve_path('preinst')) {
         if ($preinst->is_open_ok) {
-            if ($preinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($preinst->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 $self->tag(
                     'maintscript-calls-ldconfig', 'preinst'
                       # Assume it is needed if glibc does it
@@ -685,7 +685,7 @@ sub always {
     if (my $postinst = $processable->control->resolve_path('postinst')) {
         if ($postinst->is_open_ok) {
             # Decide if we call ldconfig
-            if ($postinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($postinst->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 if ($type eq 'udeb') {
                     $self->tag('udeb-postinst-must-not-call-ldconfig');
                 } else {
@@ -732,7 +732,7 @@ sub always {
 
     if (my $prerm = $processable->control->resolve_path('prerm')) {
         if ($prerm->is_open_ok) {
-            if ($prerm->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($prerm->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 $self->tag(
                     'maintscript-calls-ldconfig', 'prerm'
                       # Assume it is needed if glibc does it
@@ -743,7 +743,7 @@ sub always {
 
     if (my $postrm = $processable->control->resolve_path('postrm')) {
         if ($postrm->is_open_ok) {
-            my $contents = $postrm->slurp;
+            my $contents = $postrm->bytes;
 
             # Decide if we call ldconfig
             if ($contents =~ m/^[^\#]*\bldconfig\b/m) {
