@@ -19,29 +19,29 @@
 
 package Lintian::Group;
 
-use strict;
+use v5.20;
 use warnings;
-use v5.16;
+use utf8;
+use autodie;
 
 use Carp;
+use Cwd;
 use Devel::Size qw(total_size);
 use File::Spec;
-use IO::Async::Loop;
-use IO::Async::Routine;
 use List::Compare;
 use List::MoreUtils qw(uniq firstval);
 use Path::Tiny;
 use POSIX qw(ENOENT);
 use Time::HiRes qw(gettimeofday tv_interval);
+use Time::Piece;
+use Unicode::UTF8 qw(valid_utf8 decode_utf8);
 
 use Lintian::Processable::Binary;
 use Lintian::Processable::Buildinfo;
 use Lintian::Processable::Changes;
 use Lintian::Processable::Source;
 use Lintian::Processable::Udeb;
-use Lintian::Tags qw(tag);
-use Lintian::Unpack::Task;
-use Lintian::Util qw(internal_error get_dsc_info strip human_bytes);
+use Lintian::Util qw(get_dsc_info_from_string strip human_bytes);
 
 use constant EMPTY => q{};
 use constant SPACE => q{ };
@@ -117,62 +117,24 @@ Returns or sets the max number of jobs to be processed in parallel.
 If the limit is 0, then there is no limit for the number of parallel
 jobs.
 
-=item active
+=item processing_start
 
-Hash reference for active jobs.
-
-=item failed
-
-Array reference for failed jobs.
+=item processing_end
 
 =item cache
 
 Cache for some items.
 
-=item coll2priority
-
-Hash linking collection to priority.
-
-=item coll_priorities
-
-Hash with active jobs.
-
-=item C<collmap>
-
-Hash with active jobs.
-
-=item colls_not_scheduled
-
-Hash with active jobs.
-
-=item extra_coll
-
-Hash with active jobs.
-
 =item profile
 
 Hash with active jobs.
 
-=item queue
-
-Hash with active jobs.
-
-=item running_jobs
-
-Hash with active jobs.
-
-=item worktable
-
-Hash with active jobs.
-
 =item C<saved_direct_dependencies>
 
 =item C<saved_direct_reliants>
 
 =item C<saved_spelling_exceptions>
 
-=item C<shared_storage>
-
 =cut
 
 has pooldir => (is => 'rw', default => EMPTY);
@@ -185,28 +147,16 @@ has source => (is => 'rw');
 has udeb => (is => 'rw', default => sub{ {} });
 
 has jobs => (is => 'rw', default => 1);
-
-has coll_priorities => (is => 'rw');
+has processing_start => (is => 'rw', default => EMPTY);
+has processing_end => (is => 'rw', default => EMPTY);
 
 has cache => (is => 'rw', default => sub { {} });
-has coll2priority => (is => 'rw', default => sub { {} });
-has collmap => (is => 'rw', default => sub { {} });
 has profile => (is => 'rw', default => sub { {} });
-has running_jobs => (is => 'rw', default => sub { {} });
-has worktable => (is => 'rw', default => sub { {} });
-has active => (is => 'rw', default => sub { {} });
-has failed => (is => 'rw', default => sub { {} });
-has colls_not_scheduled => (is => 'rw', default => sub { {} });
-
-has extra_coll => (is => 'rw', default => sub { [] });
-has queue => (is => 'rw', default => sub { [] });
 
 has saved_direct_dependencies => (is => 'rw', default => sub { {} });
 has saved_direct_reliants => (is => 'rw', default => sub { {} });
 has saved_spelling_exceptions => (is => 'rw', default => sub { {} });
 
-has shared_storage => (is => 'rw', default => sub { {} });
-
 =item Lintian::Group->init_from_file (FILE)
 
 Add all processables from .changes or .buildinfo file FILE.
@@ -228,7 +178,8 @@ sub _get_processable {
     } elsif ($file =~ m/\.buildinfo$/o) {
         $processable = Lintian::Processable::Buildinfo->new;
 
-    } elsif ($file =~ m/\.deb$/o) {
+    } elsif ($file =~ m/\.d?deb$/) {
+        # in ubuntu, automatic dbgsym packages end with .ddeb
         $processable = Lintian::Processable::Binary->new;
 
     } elsif ($file =~ m/\.udeb$/o) {
@@ -255,14 +206,27 @@ sub init_from_file {
       unless defined $path;
 
     my $processable = $self->_get_processable($path);
+    return
+      unless $processable;
+
     $self->add_processable($processable);
 
     my ($type) = $path =~ m/\.(buildinfo|changes)$/;
     return
       unless defined $type;
 
-    my $info = get_dsc_info($path)
-      or internal_error("$path is not a valid $type file");
+    my $bytes = path($path)->slurp;
+
+    my $contents;
+    if(valid_utf8($bytes)) {
+        $contents = decode_utf8($bytes);
+    } else {
+        # try to proceed with nat'l encoding; stopping here breaks tests
+        $contents = $bytes;
+    }
+
+    my $info = get_dsc_info_from_string($contents)
+      or die "$path is not a valid $type file";
 
     my $dir = $path;
     if ($path =~ m,^/+[^/]++$,o){
@@ -302,9 +266,9 @@ sub init_from_file {
             exit 2;
         }
 
-        # Some file we do not care about (at least not here).
+        # only care about some files; ddeb is ubuntu dbgsym
         next
-          unless $file =~ /\.u?deb$/
+          unless $file =~ /\.(?:u|d)?deb$/
           || $file =~ m/\.dsc$/
           || $file =~ m/\.buildinfo$/;
 
@@ -322,54 +286,8 @@ Unpack this group.
 =cut
 
 sub unpack {
-    my ($self, $collmap, $action, $exit_code_ref, $OUTPUT)= @_;
+    my ($self, $OUTPUT)= @_;
 
-    my $clonedmap = $collmap->clone;
-
-    if ($self->profile) {
-        my @requested;
-        foreach my $check ($self->profile->scripts) {
-            my $script = $self->profile->get_script($check);
-            push(@requested, $script->needs_info);
-        }
-        push(@requested, @{$self->extra_coll});
-
-        # For new entries we take everything in the collmap, which is
-        # a bit too much in some cases.  Since we have cloned collmap,
-        # we might as well prune the nodes we will not need in our
-        # copy.  While not perfect, it reduces the unnecessary work
-        # rather well.
-        #
-        #  Known issue: "lintian -oC files some.dsc" should not need
-        #  to do anything because "files" is "binary, udeb"-only.
-
-        # add all ancestors; List::Compare does not need a unique list
-        push(@requested, $clonedmap->non_unique_ancestors($_)) for @requested;
-
-        # remove unneeded nodes in our copy
-        my @known = $clonedmap->known;
-        my $lc = List::Compare->new('--unsorted', \@known, \@requested);
-        $clonedmap->unlink($_)for $lc->get_Lonly;
-
-        # clonedmap should remain internally consistent
-        die 'Inconsistent collmap after deletion'
-          if $clonedmap->missing;
-    }
-
-    # Initialise our copy
-    $clonedmap->initialise;
-
-    $self->collmap($clonedmap);
-
-    my $all_ok = 1;
-
-    # Kill pending jobs, if any
-    $self->kill_jobs;
-    $self->wait_for_jobs;
-
-    $self->worktable({});
-
-    # Stop here if there is nothing for us to do
     my @processables = $self->get_processables;
     for my $processable (@processables) {
 
@@ -389,197 +307,15 @@ sub unpack {
         }
     }
 
-    my %worklists;
-    foreach my $processable (@processables) {
-
-        my $type = $processable->type;
-        my $cmap;
-
-        if (exists $self->cache->{$type}) {
-            $cmap = $self->cache->{$type}->clone;
-        } else {
-            my $collmap = $self->collmap;
-            my $cmap2 = Lintian::DepMap::Properties->new;
-            my $cond = { 'type' => $type };
-            my $coll2priority = $self->coll2priority;
-
-            foreach my $node ($collmap->known) {
-                my $script = $collmap->getp($node);
-                if (not exists($coll2priority->{$node})) {
-                    $coll2priority->{$node} = $script->priority;
-                    $self->coll_priorities(undef);
-                }
-                $cmap2->add($node, $script->needs_info($cond), $script);
-            }
-
-            $cmap2->initialise;
-
-            $self->cache->{$type} = $cmap2;
-            $cmap = $cmap2->clone;
-        }
-
-        my $needed;
-        my %wanted;
-        my @requested;
-        my $profile = $self->profile;
-        if ($profile) {
-            if ($profile) {
-
-                foreach my $check ($profile->scripts) {
-                    my $script = $profile->get_script($check);
-                    push(@requested, $script->needs_info)
-                      if $script->is_check_type($type);
-                }
-
-                push(@requested, @{$self->extra_coll});
-                @requested = uniq @requested;
-
-            } else {
-                # not new
-                @requested = $cmap->known;
-            }
-
-            while (my $check = pop @requested) {
-                my $script = $cmap->getp($check);
-                # Skip collections not relevant to us (they will never
-                # be finished and we do not want to use their
-                # dependencies if they are the only ones using them)
-                next unless $script->is_type($type);
-                $wanted{$check} = 1;
-                push @requested, $script->needs_info;
-            }
-
-            # skip it unless we need to unpack something.
-            if (%wanted) {
-                $needed = \%wanted;
-            } else {
-                $cmap = undef;
-                $needed = undef;
-            }
-        } else {
-            # if its new and $profile is undef, we have to run all
-            # of collections.  So lets exit early.
-            $needed = undef;
-        }
-
-        next unless $cmap; # nothing to do
-
-        $worklists{$processable->identifier} = {
-            'collmap' => $cmap,
-            'lab-entry' => $processable,
-            'needed' => $needed,
-        };
-    }
-
-    return 1
-      unless %worklists;
-
-    $self->worktable(\%worklists);
-
-    unless($self->coll_priorities) {
-        my @priorities
-          = sort { $self->coll2priority->{$a} <=> $self->coll2priority->{$b} }
-          keys %{$self->coll2priority};
-
-        $self->coll_priorities(\@priorities);
-    }
-
     $OUTPUT->v_msg('Unpacking packages in group ' . $self->name);
 
-    my %timers;
-    my $hook = sub {
-        $self->coll_hook($OUTPUT, $action, $exit_code_ref, \%timers, @_)
-          or $all_ok = 0;
-    };
+    my @unpack = grep { $_->can('unpack') } @processables;
 
-    $self->active({ map { $_ => 1 } keys %{$self->worktable} });
+    my $savedir = getcwd;
+    $_->unpack for @unpack;
+    chdir($savedir);
 
-    for my $check (@{$self->coll_priorities}) {
-        my %procs;
-        $procs{$_} = 1 for keys %{$self->worktable};
-        $self->colls_not_scheduled->{$check} = \%procs;
-    }
-
-    my @slices;
-
-    my $loop = IO::Async::Loop->new;
-
-    for (0..$self->jobs-1) {
-        my $task = $self->find_next_task($OUTPUT);
-        last if not $task;
-
-        my $slice = $loop->new_future;
-        push(@slices, $slice);
-
-        $self->start_task($slice, $hook, $task, $OUTPUT);
-    }
-
-    Future->wait_all(@slices)->get;
-
-    return $all_ok;
-}
-
-=item coll_hook
-
-Collection hook.
-
-=cut
-
-sub coll_hook {
-    my ($self, $OUTPUT, $action, $exit_code_ref,$timers, $task, $event,
-        $exitval)= @_;
-
-    my $coll = $task->script->name;
-    my $procid = $task->processable->identifier;
-    my $pkg_name = $task->processable->name;
-    my $pkg_type = $task->processable->type;
-
-    if ($event eq 'start') {
-        $timers->{$task->id} = [gettimeofday];
-        $OUTPUT->debug_msg(1, "Collecting info: $coll for $procid ...");
-
-        return 1;
-
-    } elsif ($event eq 'start-failed' || ($event eq 'finish' && $exitval)) {
-
-        # failed
-        my $string
-          = "collection $coll failed for $pkg_type package $pkg_name, skipping $action";
-        $string .= " error: $exitval"
-          if $exitval;
-        $OUTPUT->warning($string);
-
-        my $pkg_type = $task->processable->type;
-        if (   $pkg_type eq 'source'
-            or $pkg_type eq 'changes'
-            or $pkg_type eq 'buildinfo'){
-
-            $self->$pkg_type(undef);
-        } else {
-            my $phash = $self->$pkg_type;
-            my $id = $task->processable->identifier;
-
-            delete $phash->{$id};
-        }
-
-        $$exit_code_ref = 2;
-
-        return 0;
-
-    } elsif ($event eq 'finish') {
-
-        # success
-        my $raw_res = tv_interval($timers->{$task->id});
-        my $tres = sprintf('%.3fs', $raw_res);
-        $OUTPUT->debug_msg(1,
-            "Collection script $coll for $procid done ($tres)");
-        $OUTPUT->perf_log("$procid,coll/$coll,${raw_res}");
-
-        return 1;
-    }
-
-    # unknown event
-    return 1;
+    return;
 }
 
 =item process
@@ -589,7 +325,9 @@ Process group.
 =cut
 
 sub process {
-    my ($self, $TAGS, $exit_code_ref, $override_count, $opt, $OUTPUT)= @_;
+    my ($self, $exit_code_ref, $override_count, $opt, $OUTPUT)= @_;
+
+    $self->processing_start(gmtime->datetime);
 
     my $all_ok = 1;
 
@@ -598,21 +336,13 @@ sub process {
     my %statistics = (
         types     => {},
         severity  => {},
-        certainty => {},
         tags      => {},
         overrides => {},
     );
 
-    my @reported;
+    my @reported_tags;
 
-    foreach my $processable ($self->get_processables){
-        my $pkg_type = $processable->type;
-        my $procid = $processable->identifier;
-
-        my $path = $processable->path;
-
-        # required for global Lintian::Tags::tag usage
-        $TAGS->{current} = $path;
+    for my $processable ($self->get_processables){
 
         my $declared_overrides;
         my %used_overrides;
@@ -623,10 +353,7 @@ sub process {
         $OUTPUT->debug_msg(1,
             'Base directory for group: ' . $processable->groupdir);
 
-        my @found;
-
-        if (not $opt->{'no-override'}
-            and $self->collmap->getp('override-file')) {
+        unless ($opt->{'no-override'}) {
 
             $OUTPUT->debug_msg(1, 'Loading overrides file (if any) ...');
 
@@ -635,7 +362,7 @@ sub process {
                 die $err if not ref $err or $err->errno != ENOENT;
             }
 
-            my %alias = %{$TAGS->{profile}->aliases};
+            my %alias = %{$self->profile->known_aliases};
 
             # treat renamed tags in overrides
             for my $tagname (keys %{$declared_overrides}) {
@@ -662,17 +389,10 @@ sub process {
                 }
             }
 
-            push(@found, @{$processable->found});
-
-            # remove found tags from Processable
-            $processable->found([]);
-
             # treat ignored overrides here
             for my $tagname (keys %{$declared_overrides}) {
 
-                if ($TAGS->{profile}
-                    && !$TAGS->{profile}->is_overridable($tagname)) {
-
+                unless ($self->profile->is_overridable($tagname)) {
                     delete $declared_overrides->{$tagname};
                     $override_count->{ignored}{$tagname}++;
                 }
@@ -680,48 +400,39 @@ sub process {
 
             for my $tagname (keys %{$declared_overrides}) {
 
-                my $extras = $declared_overrides->{$tagname};
+                my $hints = $declared_overrides->{$tagname};
 
-                # set the use count to zero for each $extra
-                $used_overrides{$tagname}{$_} = 0 for keys %{$extras};
+                # set the use count to zero for each hint
+                $used_overrides{$tagname}{$_} = 0 for keys %{$hints};
             }
         }
 
         # Filter out the "lintian" check if present - it does no real harm,
         # but it adds a bit of noise in the debug output.
-        my @scripts = sort $self->profile->scripts;
-        @scripts = grep { $_ ne 'lintian' } @scripts;
+        my @checknames
+          = grep { $_ ne 'lintian' } $self->profile->enabled_checks;
+        my @checkinfos = map { $self->profile->get_checkinfo($_) } @checknames;
 
-        foreach my $script (@scripts) {
-            my $cs = $self->profile->get_script($script);
-            my $check = $cs->name;
+        for my $checkinfo (@checkinfos) {
+            my $checkname = $checkinfo->name;
             my $timer = [gettimeofday];
 
             # The lintian check is done by this frontend and we
             # also skip the check if it is not for this type of
             # package.
             next
-              if !$cs->is_check_type($pkg_type);
+              if !$checkinfo->is_check_type($processable->type);
 
-            $OUTPUT->debug_msg(1, "Running check: $check on $procid  ...");
-            eval {$cs->run_check($processable, $self);};
+            my $procid = $processable->identifier;
+            $OUTPUT->debug_msg(1, "Running check: $checkname on $procid  ...");
+
+            eval {$checkinfo->run_check($processable, $self);};
             my $err = $@;
             my $raw_res = tv_interval($timer);
 
-            push(@found, @{$processable->found});
-
-            # remove found tags from Processable
-            $processable->found([]);
-
-            # add any from the global queue
-            push(@found, @{$TAGS->{queue}});
-
-            # empty the global queue
-            $TAGS->{queue} = [];
-
             if ($err) {
                 print STDERR $err;
-                print STDERR "internal error: cannot run $check check",
+                print STDERR "internal error: cannot run $checkname check",
                   " on package $procid\n";
                 $OUTPUT->warning("skipping check of $procid");
                 $$exit_code_ref = 2;
@@ -731,39 +442,37 @@ sub process {
             }
             my $tres = sprintf('%.3fs', $raw_res);
             $OUTPUT->debug_msg(1,
-                "Check script $check for $procid done ($tres)");
-            $OUTPUT->perf_log("$procid,check/$check,${raw_res}");
+                "Check script $checkname for $procid done ($tres)");
+            $OUTPUT->perf_log("$procid,check/$checkname,${raw_res}");
         }
 
-        undef $TAGS->{current};
+        my $knownlc
+          = List::Compare->new([map { $_->name } @{$processable->tags}],
+            [$self->profile->known_tags]);
+        my @unknown_tagnames = $knownlc->get_Lonly;
+        croak 'tried to issue unknown tags: ' . join(SPACE, @unknown_tagnames)
+          if @unknown_tagnames;
 
-        my @keep;
+        # remove disabled tags
+        my @enabled_tags
+          = grep { $self->profile->tag_is_enabled($_->name) }
+          @{$processable->tags};
+        $processable->tags(\@enabled_tags);
 
-        for my $tag (@found) {
-
-            # Note, we get the known as it will be suppressed by
-            # $self->suppressed below if the tag is not enabled.
-            my $taginfo = $self->profile->get_tag($tag->name, 1);
-            croak 'tried to issue unknown tag ' . $tag->name
-              unless $taginfo;
-
-            $tag->info($taginfo);
-
-            next
-              if $TAGS->suppressed($tag->name);
+        my @keep_tags;
+        for my $tag (@{$processable->tags}) {
 
             my $override;
-            my $extra = $tag->extra;
 
             my $tag_overrides= $declared_overrides->{$tag->name};
             if ($tag_overrides) {
 
                 # do not use EMPTY; hash keys literal
-                # empty extra in specification matches all
+                # empty hint in specification matches all
                 $override = $tag_overrides->{''};
 
-                # matches 'extra' exactly
-                $override = $tag_overrides->{$extra}
+                # matches hint exactly
+                $override = $tag_overrides->{$tag->hint}
                   unless $override;
 
                 # look for patterns
@@ -773,7 +482,7 @@ sub process {
                       keys %{$tag_overrides};
 
                     my $match= firstval {
-                        $extra =~ m/^$tag_overrides->{$_}{pattern}\z/
+                        $tag->hint =~ m/^$tag_overrides->{$_}{pattern}\z/
                     }
                     @candidates;
 
@@ -781,59 +490,58 @@ sub process {
                       if $match;
                 }
 
-                $used_overrides{$tag->name}{$override->{extra}}++
+                $used_overrides{$tag->name}{$override->{hint}}++
                   if $override;
             }
 
             $tag->override($override);
 
-            push(@keep, $tag);
+            push(@keep_tags, $tag);
         }
 
+        $processable->tags(\@keep_tags);
+
         # look for unused overrides
         # should this not iterate over $tag_overrides instead?
-        for my $tagname (sort keys %used_overrides) {
+        for my $tagname (keys %used_overrides) {
 
             next
-              if $TAGS->suppressed($tagname);
+              unless $self->profile->tag_is_enabled($tagname);
 
             my $tag_overrides = $used_overrides{$tagname};
 
-            for my $extra (sort keys %{$tag_overrides}) {
+            for my $hint (keys %{$tag_overrides}) {
 
                 next
-                  if $tag_overrides->{$extra};
+                  if $tag_overrides->{$hint};
 
                 # cannot be overridden or suppressed
-                my $tag = Lintian::Tag::Standard->new;
-                $tag->name('unused-override');
-                $tag->arguments([$tagname, $extra]);
-
-                my $taginfo = $self->profile->get_tag('unused-override', 1);
-                $tag->info($taginfo);
-
-                push(@keep, $tag);
+                $processable->tag('unused-override', $tagname, $hint);
 
                 $override_count->{unused}++;
             }
         }
 
         # associate all tags with processable
-        $_->processable($processable) for @keep;
+        $_->processable($processable) for @{$processable->tags};
 
-        push(@reported, @keep);
-        @keep = ();
+        push(@reported_tags, @{$processable->tags});
+
+        # remove circular references
+        $processable->tags([]);
     }
 
-    for my $tag (@reported) {
+    # copy tag specifications into tags
+    $_->info($self->profile->get_taginfo($_->name)) for @reported_tags;
+
+    for my $tag (@reported_tags) {
 
         my $record = \%statistics;
         $record = $statistics{overrides}
           if $tag->override;
 
         $record->{tags}{$tag->name}++;
-        $record->{severity}{$tag->info->severity}++;
-        $record->{certainty}{$tag->info->certainty}++;
+        $record->{severity}{$tag->info->effective_severity}++;
 
         my $code = $tag->info->code;
         $code = 'X' if $tag->info->experimental;
@@ -849,30 +557,60 @@ sub process {
     # Report override statistics.
     unless ($opt->{'no-override'} || $opt->{'show-overrides'}) {
 
-        my $errors = $statistics{overrides}{types}{E} || 0;
-        my $warnings = $statistics{overrides}{types}{W} || 0;
-        my $info = $statistics{overrides}{types}{I} || 0;
+        my $errors = $statistics{overrides}{types}{E} // 0;
+        my $warnings = $statistics{overrides}{types}{W} // 0;
+        my $info = $statistics{overrides}{types}{I} // 0;
 
         $override_count->{errors} += $errors;
         $override_count->{warnings} += $warnings;
         $override_count->{info} += $info;
     }
 
-    my @print;
+    # discard disabled tags
+    @reported_tags
+      = grep { $self->profile->tag_is_enabled($_->name) } @reported_tags;
 
-    for my $tag (@reported) {
+    # discard experimental tags
+    @reported_tags = grep { !$_->info->experimental } @reported_tags
+      unless $opt->{'display-experimental'};
 
-        next
-          if defined $tag->override
-          && !$TAGS->{show_overrides};
+    # discard overridden tags
+    @reported_tags = grep { !defined $_->override } @reported_tags
+      unless $opt->{'show-overrides'};
 
-        next
-          unless $TAGS->displayed($tag->name);
+    # discard outside the selected display level
+    @reported_tags
+      = grep { $self->profile->display_level_for_tag($_->name) }@reported_tags;
 
-        push(@print, $tag);
+    my $reference_limit = $opt->{'display-source'} // [];
+    if (@{$reference_limit}) {
+
+        my @topic_tags;
+        for my $tag (@reported_tags) {
+            my @references = split(/,/, $tag->info->references);
+
+            # retain the first word
+            s/^([\w-]+)\s.*/$1/ for @references;
+
+            # remove anything in parentheses at the end
+            s/\(\S+\)$// for @references;
+
+            # check if tag refers to the selected references
+            my $referencelc
+              = List::Compare->new(\@references, $reference_limit);
+            next
+              unless $referencelc->get_intersection;
+
+            push(@topic_tags, $tag);
+        }
+
+        @reported_tags = @topic_tags;
     }
 
-    $OUTPUT->issue_tags(\@print, [$self->get_processables]);
+    # put tags back into their respective processables
+    push(@{$_->processable->tags}, $_) for @reported_tags;
+
+    $self->processing_end(gmtime->datetime);
 
     my $raw_res = tv_interval($timer);
     my $tres = sprintf('%.3fs', $raw_res);
@@ -882,7 +620,7 @@ sub process {
 
     if ($opt->{'debug'} > 2) {
 
-        # supress warnings without reliable sizes
+        # suppress warnings without reliable sizes
         $Devel::Size::warn = 0;
 
         my $pivot = ($self->get_processables)[0];
@@ -952,13 +690,11 @@ added.
 sub add_processable{
     my ($self, $processable) = @_;
 
-    my $pkg_type = $processable->type;
-
     if ($processable->tainted) {
         warn(
             sprintf(
                 "warning: tainted %1\$s package '%2\$s', skipping\n",
-                $pkg_type, $processable->name
+                $processable->type, $processable->name
             ));
         return 0;
     }
@@ -973,44 +709,41 @@ sub add_processable{
     croak 'Please set pool directory first.'
       unless $self->pooldir;
 
-    croak "Not a supported type ($pkg_type)"
-      unless exists $SUPPORTED_TYPES{$pkg_type};
+    croak 'Not a supported type (' . $processable->type . ')'
+      unless exists $SUPPORTED_TYPES{$processable->type};
 
     my $dir = $self->_pool_path($processable);
 
     $processable->groupdir($dir);
 
-    if ($pkg_type eq 'changes') {
-        internal_error("Cannot add another $pkg_type file")
+    if ($processable->type eq 'changes') {
+        die 'Cannot add another ' . $processable->type . ' file'
           if $self->changes;
         $self->changes($processable);
 
-    } elsif ($pkg_type eq 'buildinfo') {
+    } elsif ($processable->type eq 'buildinfo') {
         # Ignore multiple .buildinfo files; use the first one
         $self->buildinfo($processable)
           unless $self->buildinfo;
 
-    } elsif ($pkg_type eq 'source'){
-        internal_error('Cannot add another source package')
+    } elsif ($processable->type eq 'source'){
+        die 'Cannot add another source package'
           if $self->source;
         $self->source($processable);
 
     } else {
-        my $phash;
+        my $type = $processable->type;
+        die 'Unknown type ' . $type
+          unless $type eq 'binary' || $type eq 'udeb';
+
+        # check for duplicate; should be rewritten with arrays
         my $id = $processable->identifier;
-        internal_error("Unknown type $pkg_type")
-          unless ($pkg_type eq 'binary' or $pkg_type eq 'udeb');
-        $phash = $self->$pkg_type;
-
-        # duplicate ?
         return 0
-          if exists $phash->{$id};
+          if exists $self->$type->{$id};
 
-        $phash->{$id} = $processable;
+        $self->$type->{$id} = $processable;
     }
 
-    $processable->shared_storage($self->shared_storage);
-
     return 1;
 }
 
@@ -1077,7 +810,7 @@ sub get_processables {
         return values %{$self->$type}
           if $type eq 'binary'
           or $type eq 'udeb';
-        internal_error("Unknown type of processable: $type");
+        die "Unknown type of processable: $type";
     }
     # We return changes, dsc, buildinfo, debs and udebs in that order,
     # because that is the order lintian used to process a changes
@@ -1289,255 +1022,6 @@ sub clear_cache {
     return;
 }
 
-=item find_next_task
-
-Find next task.
-
-=cut
-
-sub find_next_task {
-    my ($self, $OUTPUT) = @_;
-
-    my @coll_priorities = @{$self->coll_priorities};
-
-    unless (@{$self->queue}) {
-
-        for (my $i = 0; $i < @coll_priorities ; $i++) {
-
-            my $check = $coll_priorities[$i];
-            my $script = $self->collmap->getp($check);
-            my $unscheduled = $self->colls_not_scheduled->{$check};
-
-            foreach
-              my $procid (grep { $unscheduled->{$_} } keys %{$self->active}) {
-                my $wlist = $self->worktable->{$procid};
-                my $cmap = $wlist->{'collmap'};
-
-                next
-                  unless $cmap->selectable($check);
-
-                my $processable = $wlist->{'lab-entry'};
-                my $needed = $wlist->{'needed'};
-
-                delete $unscheduled->{$procid};
-
-                # current type?
-                unless ($script->is_type($processable->type)) {
-                    $cmap->satisfy($check);
-                    next;
-                }
-
-                # Check if its actually on our TODO list.
-                if (defined $needed and not exists $needed->{$check}) {
-                    $cmap->satisfy($check);
-                    next;
-                }
-
-                # collect info
-                $cmap->select($check);
-                $OUTPUT->debug_msg(3, "READY $check-$procid")
-                  if $OUTPUT->debug;
-
-                my $task = Lintian::Unpack::Task->new;
-                $task->id($check . HYPHEN . $procid);
-                $task->script($script);
-                $task->processable($processable);
-                $task->cmap($cmap);
-                push(@{$self->queue}, $task);
-
-               # If we are dealing with the highest priority type of task, then
-               # keep filling the cache (i.e. $i == 0).  Otherwise, stop here
-               # to avoid priority inversion due to filling the queue with
-               # unimportant tasks.
-                last if $i;
-            }
-
-            unless (keys %{$unscheduled}) {
-                $OUTPUT->debug_msg(3,
-                    "DISCARD $check (all instances have been scheduled)")
-                  if $OUTPUT->debug;
-                splice(@coll_priorities, $i, 1);
-                $i--;
-            }
-
-            last if @{$self->queue};
-        }
-    }
-
-    if (@{$self->queue}) {
-        $OUTPUT->debug_msg(4,
-                'QUEUE non-empty with '
-              . scalar(@{$self->queue})
-              . ' item(s).  Taking one.')
-          if $OUTPUT->debug;
-    }
-
-    return shift @{$self->queue}
-      if @{$self->queue};
-
-    return;
-}
-
-=item start_task
-
-Start task.
-
-=cut
-
-sub start_task {
-    my ($self, $slice, $hook, $task, $OUTPUT) = @_;
-
-    my $cmap = $task->cmap;
-
-    my $loop = IO::Async::Loop->new;
-    my $debug_enabled = $OUTPUT->debug;
-
-    $OUTPUT->debug_msg(3, 'START ' . $task->id);
-    my $pid = -1;
-
-    my $future = $loop->new_future;
-
-    $hook->($task, 'start')
-      if $hook;
-
-    eval {
-
-        $pid = $loop->fork(
-            code  => sub {
-
-                # fixed upstream in 0.73
-                undef($IO::Async::Loop::ONE_TRUE_LOOP);
-
-                my $check = $task->script->name;
-                my $procid = $task->processable->identifier;
-
-                my $package = $task->processable->name;
-                my $type = $task->processable->type;
-                my $groupdir = $task->processable->groupdir;
-
-                # change the process name; possible overwritten by exec
-                $0 = "$check (processing $procid)";
-
-                my $ret = 0;
-                eval {$task->script->collect($package, $type, $groupdir);};
-                if ($@) {
-                    print STDERR $@;
-                    $ret = 2;
-                }
-                POSIX::_exit($ret);
-            },
-
-            on_exit  => sub {
-                my ($pid, $status) = @_;
-
-                delete $self->running_jobs->{$future};
-
-                $OUTPUT->debug_msg(3, 'FINISH ' . $task->id . " ($status)");
-
-                $hook->($task, 'finish', $status)
-                  if $hook;
-
-                my $check = $task->script->name;
-                my $procid = $task->processable->identifier;
-
-                if ($status) {
-                    # failed ...
-                    $self->failed->{$procid} = 1;
-                    delete $self->active->{$procid};
-                }else {
-                    # The collection was success
-                    $cmap->satisfy($check);
-                    # If the entry is marked as failed, don't break the loop
-                    # for it.
-                    $self->active->{$procid} = 1
-                      unless $self->failed->{$procid}
-                      || !$cmap->selectable;
-                }
-
-                $future->done("Script $check for ". $procid. ' finished');
-
-            });
-    };
-
-    $future->on_ready(
-        sub {
-            my $task = $self->find_next_task($OUTPUT);
-            $slice->done('No more tasks')
-              unless $task;
-            $self->start_task($slice, $hook, $task, $OUTPUT)
-              if $task;
-
-            my $debug_enabled = $OUTPUT->debug;
-            if ($debug_enabled) {
-                my @ids = map { $_->{id} } values %{$self->running_jobs};
-                my $queue = join(', ', sort @ids);
-                $OUTPUT->debug_msg(3, "RUNNING QUEUE: $queue");
-            }
-        });
-
-    if ($hook) {
-        my $err = $@;
-        $hook->($task, 'failed', $err)
-          if $pid == -1;
-    }
-
-    $self->running_jobs->{$future} = { id => $task->id, pid => $pid };
-
-    return;
-}
-
-=item wait_for_jobs
-
-Block and wait for all running jobs to terminate.  Usually this is not
-needed unless process_tasks was interrupted somehow.
-
-=cut
-
-sub wait_for_jobs {
-    my ($self) = @_;
-
-    my @futures = keys %{$self->running_jobs};
-    Future->wait_all(@futures)->get;
-
-    $self->running_jobs({});
-    return;
-}
-
-=item kill_jobs
-
-Forcefully terminate all running jobs.  Usually this is not needed
-unless process_tasks was interrupted somehow.
-
-=cut
-
-sub kill_jobs {
-    my ($self) = @_;
-
-    my @pids = map { $_->{pid} } values %{$self->running_jobs};
-    if (@pids) {
-        kill('TERM', @pids);
-        kill('KILL', @pids);
-    }
-
-    $self->running_jobs({});
-    return;
-}
-
-=item DEMOLISH
-
-Moo destructor.
-
-=cut
-
-sub DEMOLISH {
-    my ($self, $in_global_destruction) = @_;
-
-    # kill any remaining jobs.
-    $self->kill_jobs;
-
-    return;
-}
-
 =back
 
 =head1 AUTHOR
